@@ -1,5 +1,5 @@
 graph <- function(matrix.pval, xi = c(seq(0.001,0.009,by=0.001),seq(0.010,0.985,by=0.005),seq(0.990,0.999,by=0.001)),
-			      type = c("pvalue.plot", "pvalue.discrepancy", "size.power")) {
+			      type = c("pvalue.plot", "pvalue.discrepancy", "size.power"), center = FALSE, scale = FALSE) {
 	
   type <- match.arg(type)
 		
@@ -25,7 +25,7 @@ graph <- function(matrix.pval, xi = c(seq(0.001,0.009,by=0.001),seq(0.010,0.985,
 
 ### Size-power curves
   if (type == "size.power") {
-    pnull <- many.pval(stat.indices, law.index=null.dist, n, M, N, alter, law.pars=NULL, parstats, null.dist, method)$pvals
+    pnull <- many.pval(stat.indices, law.index=null.dist, n, M, N, alter, law.pars=NULL, parstats, null.dist, method, center = center, scale = scale)$pvals
     Fxnull <- calcFx(pnull, xi)
     plot.sizepower(Fxi,Fxnull)
   }
