@@ -5,7 +5,7 @@
 // WARNING: getname[0] should never be set to 1 in this file because it has the side effect to change the values of the parameters of either the laws or the stats.
 
 #include <iostream>
-using namespace std;
+// // using namespace std; // Pierre modified this on 22/12/2015
 
 #include <R.h>
 #include "Rmath.h"
@@ -21,9 +21,31 @@ static char *sfunction;
 
   // Generation of the sample
   
-  int gensample(int law, int *xlen, double *x, char **name, int *getname, double *params, int *nbparams, int *setseed) {
+  int gensample(int law, int *xlen, double *x, char **name, int *getname, double *params, int *nbparams, int *setseed, int *center, int *scale) {
     
+    int i, n = xlen[0];
+    double meanX = 0.0, varX = 0.0, sdX;
+
     (*lawfunc[law-1])(xlen,x,name,getname,params,nbparams,setseed);
+
+    if (scale[0] == 1) {
+      for (i = 0; i <= (n - 1); i++) meanX = meanX + x[i];
+      meanX = meanX / (double)n;
+      for (i = 0; i <= (n - 1); i++) varX = varX + R_pow(x[i], 2.0);
+      varX = ((double)n) * (varX / (double)n - R_pow(meanX, 2.0)) / (double)(n - 1); 
+      sdX = sqrt(varX);
+      if (center[0] == 1) {
+	for (i = 0; i <= (n - 1); i++) x[i] = (x[i] - meanX) / sdX;
+      } else {
+	for (i = 0; i <= (n - 1); i++) x[i] = x[i] / sdX;
+      }
+    } else {
+      if (center[0] == 1) {
+	for (i = 0; i <= (n - 1); i++) meanX = meanX + x[i];
+	meanX = meanX / (double)n;
+	for (i = 0; i <= (n - 1); i++) x[i] = x[i] - meanX;
+      }
+    }
     
     return(1);
   
@@ -119,9 +141,9 @@ static char *sfunction;
   
   // Computation of the power of the test statistic
   void powcompeasy(int *M, double *params, int *ncolparams, int *decision, int *decisionlen, //char **lawnames, char **statnames, 
-int *modelnum, char** funclist, double *thetavec, double *xvec, int *p, int *np) {
+int *modelnum, char** funclist, double *thetavec, double *xvec, int *p, int *np, int *center, int *scale) {
     
-    int gensample(int law, int *xlen, double *x, char **name1, int *getname, double *params, int *nbparams, int *setseed);
+    int gensample(int law, int *xlen, double *x, char **name1, int *getname, double *params, int *nbparams, int *setseed, int *center, int *scale);
     void statcompute(int stat, double *x, int *xlen, double *level, int *nblevel, char **name2, int *getname, double *statistic, 
 		     int *pvalcomp, double *pvalue, double *critvalL, double *critvalR, int *usecrit, int *alter, int *decision, double *paramstat, int *nbparamstat);
     int model(int modelnum, char** funclist, double *thetavec, double *xvec, int *xlen, double *x, int *p, int *np);
@@ -140,9 +162,11 @@ int *modelnum, char** funclist, double *thetavec, double *xvec, int *p, int *np)
     char **name1, **name2;
     name1 = new char*[50];
     name2 = new char*[50];
-    for (i=1;i<=50;i++) {
-      *(name1+i-1) =  new char[1];
-      *(name2+i-1) =  new char[1];
+    for (i = 0; i < 50; i++) {
+      name1[i] =  new char[1];
+      name2[i] =  new char[1];
+      name1[i][0] = ' ';
+      name2[i][0] = ' ';
     }
     getname = new int[1];
     int *decisiontmp, *nblevel, *nbparlaw, *nbparstat;
@@ -185,15 +209,15 @@ int *modelnum, char** funclist, double *thetavec, double *xvec, int *p, int *np)
       for (i=0;i<nbparstat[0];i++) parstat[i] = params[(ncolparams[0])*(row-1)+ 14 + i];
 
       x = new double[n];
-      for (j=1;j<=n;j++) *(x+j-1) = 0.0;
+      for (j = 0; j < n; j++) x[j] = 0.0;
       xlen = new int[1];
-      *(xlen+0) = n;
+      xlen[0] = n;
 
       getname[0] = 0;
      if (M[0]>1) {
 	for (i=1;i<=*(M+0);i++) { // on part la simul, sans refaire la 1ere iteration!
 	  
-	  gensample(law,xlen,x,name1,getname,parlaw,nbparlaw,setseed); // on génère l'échantillon
+	  gensample(law,xlen,x,name1,getname,parlaw,nbparlaw,setseed,center,scale); // on génère l'échantillon
 	  model(modelnum[0],funclist,thetavec,xvec,xlen,x,p,np);  // on applique le modèle
  
 	  statcompute(stat, x, xlen, level, nblevel, name2, getname, statistic,pvalcomp,pvalue,critvalL,critvalR,usecrit,alter,decisiontmp,parstat,nbparstat);
@@ -210,10 +234,6 @@ int *modelnum, char** funclist, double *thetavec, double *xvec, int *p, int *np)
       params[(ncolparams[0])*(row-1)+12] = parlaw[3];
       params[(ncolparams[0])*(row-1)+13] = (int)nbparstat[0];
       for (i=0;i<nbparstat[0];i++) params[(ncolparams[0])*(row-1)+ 14 + i] = parstat[i];
-
-
-
-
     
       //On libere de la memoire
       delete[] x;
@@ -232,9 +252,9 @@ int *modelnum, char** funclist, double *thetavec, double *xvec, int *p, int *np)
     
 
     //On libere de la memoire
-    for (i=1;i<=50;i++) {
-      delete[] *(name1+i-1);
-      delete[] *(name2+i-1);
+    for (i = 0; i < 50; i++) {
+      delete[] name1[i];
+      delete[] name2[i];
     }
     delete[] name1;
     delete[] name2;
@@ -253,9 +273,9 @@ int *modelnum, char** funclist, double *thetavec, double *xvec, int *p, int *np)
 
   // computation of all the statistic values in order to obtain the critical values
   void compquant(int *n, int *law, int *stat, int *M, double *statvec, //char **lawname, char **statname, 
-		 int *nbparlaw, double *parlaw, int *nbparstat, double *parstat, int *modelnum, char** funclist, double *thetavec, double *xvec, int *p, int *np) {
+		 int *nbparlaw, double *parlaw, int *nbparstat, double *parstat, int *modelnum, char** funclist, double *thetavec, double *xvec, int *p, int *np, int *center, int *scale) {
 
-    int gensample(int law, int *xlen, double *x, char **name1, int *getname, double *params, int *nbparams, int *setseed);
+    int gensample(int law, int *xlen, double *x, char **name1, int *getname, double *params, int *nbparams, int *setseed, int *center, int *scale);
     void statcompute(int stat, double *x, int *xlen, double *level, int *nblevel, char **name, int *getname, double *statistic, int *pvalcomp,double *pvalue, double *critvalL, double *critvalR, int *usecrit, int *alter, int *decision, double *paramstat, int *nbparamstat);
     int model(int modelnum, char** funclist, double *thetavec, double *xvec, int *xlen, double *x, int *p, int *np);
     
@@ -300,9 +320,11 @@ int *modelnum, char** funclist, double *thetavec, double *xvec, int *p, int *np)
     char **name1, **name2;
     name1 = new char*[50];
     name2 = new char*[50];
-    for (i=1;i<=50;i++) {
-      *(name1+i-1) =  new char[1];
-      *(name2+i-1) =  new char[1];
+    for (i = 0; i < 50; i++) {
+      name1[i] =  new char[1];
+      name2[i] =  new char[1];
+      name1[i][0] = ' ';
+      name2[i][0] = ' ';
     }
 
     int *setseed;
@@ -315,7 +337,7 @@ int *modelnum, char** funclist, double *thetavec, double *xvec, int *p, int *np)
       
       x = new double[*(n+0)];
       for (j=1;j<=*(n+0);j++) *(x+j-1) = 0.0;
-      gensample(*(law+0),n,x,name1,getname,parlaw,nbparlaw,setseed);
+      gensample(*(law+0),n,x,name1,getname,parlaw,nbparlaw,setseed,center,scale);
       model(modelnum[0],funclist,thetavec,xvec,n,x,p,np);   // on applique le modèle
     
       statcompute(*(stat+0), x, n, level, nblevel, name2, getname, statistic, pvalcomp,pvalue,critvalL,critvalR,usecrit,alter,decisiontmp,parstat,nbparstat);
@@ -329,9 +351,9 @@ int *modelnum, char** funclist, double *thetavec, double *xvec, int *p, int *np)
 	      }
 
     //On libere de la memoire
-    for (i=1;i<=50;i++) {
-      delete[] *(name1+i-1);
-      delete[] *(name2+i-1);
+    for (i = 0; i < 50; i++) {
+      delete[] name1[i];
+      delete[] name2[i];
     }
     delete[] name1;
     delete[] name2;
@@ -356,39 +378,48 @@ int *modelnum, char** funclist, double *thetavec, double *xvec, int *p, int *np)
 
   // Computation of the power of the test statistic
   void powcompfast(int *M, int *vectlaws, int *lawslen, int *vectn, int *vectnlen, int *vectstats, int *statslen, int *decision, int *decisionlen, double *level, int *nblevel,
-double *critvalL, double *critvalR, int *usecrit, int *alter, int *nbparlaw, double *parlaw, int *nbparstat, double *parstat, int *modelnum, char** funclist, double *thetavec, double *xvec, int *p, int *np) {
+		   double *critvalL, double *critvalR, int *usecrit, int *alter, int *nbparlaw, double *parlaw, int *nbparstat, double *parstat, int *modelnum, char** funclist, 
+		   double *thetavec, double *xvec, int *p, int *np, int *center, int *scale, int *compquant) {
 
 
-    int gensample(int law, int *xlen, double *x, char **name1, int *getname, double *params, int *nbparams, int *setseed);
-    void statcompute(int stat, double *x, int *xlen, double *level, int *nblevel, char **name2, int *getname, double *statistic, int *pvalcomp,double *pvalue, double *critvalL, double *critvalR, int *usecrit, int *alter, int *decision, double *paramstat, int *nbparamstat);
+    // Warning: When compquant[0] == 1, critvalL should be initialized with lawslen[0] * M[0] * vectnlen[0] * statslen[0] double values since it will contain (when output)
+    // all the test statistics generated. Most probably, lawslen[0] should be equal to 1.
+
+    // Il faut permuter M et law ci-dessous et aussi dans les fichiers R concernés (ainsi que dans la fonction print)!!
+
+    int gensample(int law, int *xlen, double *x, char **name1, int *getname, double *params, int *nbparams, int *setseed, int *center, int *scale);
+    void statcompute(int stat, double *x, int *xlen, double *level, int *nblevel, char **name2, int *getname, double *statistic, 
+		     int *pvalcomp,double *pvalue, double *critvalL, double *critvalR, int *usecrit, int *alter, int *decision, double *paramstat, int *nbparamstat);
     int model(int modelnum, char** funclist, double *thetavec, double *xvec, int *xlen, double *x, int *p, int *np);
    
-    double *statistic, *pvalue; // POUR L'INSTANT JE N'EN FAIT RIEN DE statistic et de pvalue!! Si je veux les récupérer dans R il faudra faire des modifs!! A voir ...
+    double *statistic, *pvalue; // POUR L'INSTANT JE N'EN FAIT RIEN DE pvalue!! Si je veux les récupérer dans R il faudra faire des modifs!! A voir ...
     int *pvalcomp;
     statistic = new double[1];
     pvalue = new double[1];
     pvalcomp = new int[1];
-    *(statistic+0) = 0.0;
-    *(pvalue+0) = 0.0;
-    pvalcomp[0] = 1;
-
+    statistic[0] = 0.0;
+    pvalue[0] = 0.0;
+    if (compquant[0] == 1) pvalcomp[0] = 0; else pvalcomp[0] = 1;
 
+    int indtmp = 0;
 
     int i, n, law, stat, j, *xlen, *getname;
     double *x;
     getname = new int[1];
+
     char **name1, **name2;
     name1 = new char*[50];
     name2 = new char*[50];
-    for (i=1;i<=50;i++) {
-      *(name1+i-1) =  new char[1];
-      *(name2+i-1) =  new char[1];
+    for (i = 0; i < 50; i++) {
+      name1[i] =  new char[1];
+      name2[i] =  new char[1];
+      name1[i][0] = ' ';
+      name2[i][0] = ' ';
     }
 
     int *decisiontmp;
     decisiontmp = new int[nblevel[0]];
-    for (i=1;i<=(nblevel[0]);i++) decisiontmp[i-1] = 0;
-
+    for (i = 0; i < nblevel[0]; i++) decisiontmp[i] = 0;
 
     int *setseed;
     setseed = new int[1];
@@ -396,19 +427,18 @@ double *critvalL, double *critvalR, int *usecrit, int *alter, int *nbparlaw, dou
     GetRNGstate();
 
 
-
     int maxn = vectn[0]; // maximum of vectn values
-    for (i=2;i<=vectnlen[0];i++) if (maxn < vectn[i-1]) maxn = vectn[i-1];
+    for (i = 1; i < vectnlen[0]; i++) if (maxn < vectn[i]) maxn = vectn[i];
 
     x = new double[maxn]; // on reserve un vecteur de taille (max des n dans vectn)
-    for (j=1;j<=maxn;j++) *(x+j-1) = 0.0;
+    for (j = 0; j < maxn; j++) x[j] = 0.0;
     xlen = new int[1];
 
     int *altertmp, *usecrittmp, *nbparlawtmp, *nbparstattmp;
     altertmp = new int[1];
     usecrittmp = new int[1];
     nbparlawtmp = new int[1];
-	nbparstattmp = new int[1];
+    nbparstattmp = new int[1];
 	
     double *critvalLtmp, *critvalRtmp, *parlawtmp, *parstattmp;
     critvalLtmp = new double[nblevel[0]];
@@ -417,8 +447,8 @@ double *critvalL, double *critvalR, int *usecrit, int *alter, int *nbparlaw, dou
 	
     int t;
     //    int m;
-    int kmax=0; 		// kmax = parstats.len.max in powcomp-fast.R
-    for (t=0;t<=(statslen[0]-1);t++) {
+    int kmax = 0; 		// kmax = parstats.len.max in powcomp-fast.R
+    for (t = 0; t < statslen[0]; t++) {
       if (kmax <= nbparstat[t]) {
 	kmax = nbparstat[t];
       } //else kmax = kmax;
@@ -429,65 +459,72 @@ double *critvalL, double *critvalR, int *usecrit, int *alter, int *nbparlaw, dou
     int stlen1, stlen2;
 
     getname[0] = 0;	
-    for (i=1;i<=M[0];i++) {
 
-      //    k = 0; // indice de la simul
-      for (law=1;law<=lawslen[0];law++) {
+    for (law = 0; law < lawslen[0]; law++) {
+
+      for (i = 1; i <= M[0]; i++) {
 
 	xlen[0] = maxn; // on génère un échantillon de taille maxn de loi law 
 
-	nbparlawtmp[0] = nbparlaw[law-1];
-	parlawtmp[0] = parlaw[0+4*(law-1)];
-	parlawtmp[1] = parlaw[1+4*(law-1)];
-	parlawtmp[2] = parlaw[2+4*(law-1)];
-	parlawtmp[3] = parlaw[3+4*(law-1)];
+	nbparlawtmp[0] = nbparlaw[law];
+	parlawtmp[0] = parlaw[0 + 4 * law];
+	parlawtmp[1] = parlaw[1 + 4 * law];
+	parlawtmp[2] = parlaw[2 + 4 * law];
+	parlawtmp[3] = parlaw[3 + 4 * law];
 	
-	gensample(*(vectlaws+law-1),xlen,x,name1,getname,parlawtmp,nbparlawtmp,setseed);
-	model(modelnum[0],funclist,thetavec,xvec,xlen,x,p,np);   // on applique le modèle
+	gensample(vectlaws[law], xlen, x, name1, getname, parlawtmp, nbparlawtmp, setseed, center, scale);
+	model(modelnum[0], funclist, thetavec, xvec, xlen, x, p, np);   // on applique le modèle
 	
-	if (i==1) {
-	  nbparlaw[law-1] = nbparlawtmp[0];
-	  parlaw[0+4*(law-1)] = parlawtmp[0];
-	  parlaw[1+4*(law-1)] = parlawtmp[1];
-	  parlaw[2+4*(law-1)] = parlawtmp[2];
-	  parlaw[3+4*(law-1)] = parlawtmp[3];
+	if (i == 1) {
+	  nbparlaw[law] = nbparlawtmp[0];
+	  parlaw[0 + 4 * law] = parlawtmp[0];
+	  parlaw[1 + 4 * law] = parlawtmp[1];
+	  parlaw[2 + 4 * law] = parlawtmp[2];
+	  parlaw[3 + 4 * law] = parlawtmp[3];
 	}
 
-	for (n=1;n<=vectnlen[0];n++) {
+	for (n = 0; n < vectnlen[0]; n++) {
 	  
-	  *(xlen+0) = *(vectn+n-1); // permet de ne prendre que la portion (au début) de x de taille vectn[n-1]
+	  xlen[0] = vectn[n]; // permet de ne prendre que la portion (au début) de x de taille vectn[n-1]
 	  
 	  stlen1 = 0; stlen2 = 0;
-	  for (stat=1;stat<=statslen[0];stat++) {
+	  for (stat = 0; stat < statslen[0]; stat++) {
 	    
-	    altertmp[0] = alter[stat-1];
+	    altertmp[0] = alter[stat];
 	    
-	    nbparstattmp[0] = nbparstat[stat-1];
-	    for (t=0;t<nbparstattmp[0];t++) {
-	      parstattmp[t] = parstat[t+stlen1];
+	    nbparstattmp[0] = nbparstat[stat];
+	    for (t = 0; t < nbparstattmp[0]; t++) {
+	      parstattmp[t] = parstat[t + stlen1];
 	    }
 	    stlen1 = stlen1 + nbparstattmp[0];
 	
-	    usecrittmp[0] = usecrit[n+vectnlen[0]*(stat-1)-1];
-	    for (j=1;j<=nblevel[0];j++) {
-	      critvalLtmp[j-1] = critvalL[n+nblevel[0]*vectnlen[0]*(stat-1)+vectnlen[0]*(j-1)-1];
-	      critvalRtmp[j-1] = critvalR[n+nblevel[0]*vectnlen[0]*(stat-1)+vectnlen[0]*(j-1)-1];	  
+	    usecrittmp[0] = usecrit[n + vectnlen[0] * stat];
+	    for (j = 0; j < nblevel[0]; j++) {
+	      indtmp = n + nblevel[0] * vectnlen[0] * stat + vectnlen[0] * j;
+	      critvalLtmp[j] = critvalL[indtmp];
+	      critvalRtmp[j] = critvalR[indtmp];	  
 	    }
 
 	    // decisiontmp est de longueur nblevel[0]
-	    statcompute(vectstats[stat-1], x, xlen, level, nblevel, name2, getname, statistic, pvalcomp,pvalue,critvalLtmp,critvalRtmp,usecrittmp,altertmp,decisiontmp,parstattmp,nbparstattmp);
+	    statcompute(vectstats[stat], x, xlen, level, nblevel, name2, getname, statistic, pvalcomp, pvalue, critvalLtmp, critvalRtmp, usecrittmp, altertmp, decisiontmp, parstattmp, nbparstattmp);
+	    if (compquant[0] == 1) {
+	      critvalL[i + M[0] * n + M[0] * vectnlen[0] * stat - 1] = statistic[0];
+	    }
 
-	    if (i == 1 && law == 1 && n == 1) {
 
-	      nbparstat[stat-1] = nbparstattmp[0];
-	      for (t=0;t<nbparstattmp[0];t++) {
-		parstat[t+stlen2] = parstattmp[t];
+	    if (i == 1 && law == 0 && n == 0) {
+	      nbparstat[stat] = nbparstattmp[0];
+	      for (t = 0; t < nbparstattmp[0]; t++) {
+		parstat[t + stlen2] = parstattmp[t];
 	      }
 	      stlen2 = stlen2 + nbparstattmp[0];
 	    }
 
-	    for (j=1;j<=nblevel[0];j++) {
-	      decision[stat + statslen[0]*(n-1) + statslen[0]*vectnlen[0]*(law-1) + statslen[0]*vectnlen[0]*lawslen[0]*(j-1) -1] = decision[stat + statslen[0]*(n-1) + statslen[0]*vectnlen[0]*(law-1) + statslen[0]*vectnlen[0]*lawslen[0]*(j-1) -1] + decisiontmp[j-1];
+	    if (compquant[0] == 0) {
+	      for (j = 0; j < nblevel[0]; j++) {
+		indtmp = stat + statslen[0] * n + statslen[0] * vectnlen[0] * law + statslen[0] * vectnlen[0] * lawslen[0] * j;
+		decision[indtmp] = decision[indtmp] + decisiontmp[j];
+	      }
 	    }
 	    
 	  }
@@ -499,9 +536,9 @@ double *critvalL, double *critvalR, int *usecrit, int *alter, int *nbparlaw, dou
     }
 
     //On libere de la memoire
-    for (i=1;i<=50;i++) {
-      delete[] *(name1+i-1);
-      delete[] *(name2+i-1);
+    for (i = 0; i < 50; i++) {
+      delete[] name1[i];
+      delete[] name2[i];
     }
     delete[] x;
     delete[] name1;
@@ -563,11 +600,11 @@ double *critvalL, double *critvalR, int *usecrit, int *alter, int *nbparlaw, dou
   
    
   // Computation of the p-values matrix
-  void matrixpval(int* N, int* lawindex, int *xlen, int *nbparams, double *parlaw, int *statindices, int* nbstats, int *altervec, double *parstatmultvec, int *nbparstatvec, double *res) {
+  void matrixpval(int* N, int* lawindex, int *xlen, int *nbparams, double *parlaw, int *statindices, int* nbstats, int *altervec, double *parstatmultvec, int *nbparstatvec, double *res, int *center, int *scale) {
 
     int i, j, k;
 
-    int gensample(int law, int *xlen, double *x, char **name, int *getname, double *params, int *nbparams, int *setseed);
+    int gensample(int law, int *xlen, double *x, char **name, int *getname, double *params, int *nbparams, int *setseed, int *center, int *scale);
 
     double *x; 
     x = new double[*(xlen+0)];
@@ -602,7 +639,7 @@ double *critvalL, double *critvalR, int *usecrit, int *alter, int *nbparlaw, dou
 
     char **name;          // name = rep(" ", 50)
     name = new char*[50];
-    for (i=0;i<50;i++) {
+    for (i = 0; i < 50; i++) {
       name[i] =  new char[1];
       name[i][0] = ' ';
     }
@@ -649,7 +686,7 @@ double *critvalL, double *critvalR, int *usecrit, int *alter, int *nbparlaw, dou
 
     for (j=0;j<N[0];j++) {
       
-      gensample(lawindex[0],xlen,x,name,getname,params,nbparams,setseed);  // We retrieve x
+      gensample(lawindex[0],xlen,x,name,getname,params,nbparams,setseed,center,scale);  // We retrieve x
       
       cmpt = 0;
 
@@ -711,11 +748,11 @@ double *critvalL, double *critvalR, int *usecrit, int *alter, int *nbparlaw, dou
 
 
   // Computation of the p-values matrix using Monte-Carlo
-  void matrixpvalMC(int *n, int *lawindex, int* nbstats, int *M, int *statindices, int *nbparstatvec, double *parstatmultvec, char** funclist, int *N, int *nulldist, int *nbparams, int *altervec, double *parstat, int *nbparstat, double *res) {
+  void matrixpvalMC(int *n, int *lawindex, int* nbstats, int *M, int *statindices, int *nbparstatvec, double *parstatmultvec, char** funclist, int *N, int *nulldist, int *nbparams, int *altervec, double *parstat, int *nbparstat, double *res, int *center, int *scale) {
 
     void compquant(int *n, int *law, int *stat, int *M, double *statvec,// char **lawname, char **statname, 
-int *nbparlaw, double *parlaw, int *nbparstat, double *parstat, int *modelnum, char** funclist, double *thetavec, double *xvec, int *p, int *np);
-    int gensample(int law, int *xlen, double *x, char **name, int *getname, double *params, int *nbparams, int *setseed);
+int *nbparlaw, double *parlaw, int *nbparstat, double *parstat, int *modelnum, char** funclist, double *thetavec, double *xvec, int *p, int *np, int *center, int *scale);
+    int gensample(int law, int *xlen, double *x, char **name, int *getname, double *params, int *nbparams, int *setseed, int *center, int *scale);
 
 
     int i, j, k;
@@ -790,11 +827,11 @@ int *nbparlaw, double *parlaw, int *nbparstat, double *parstat, int *modelnum, c
 	for (k=0;k<nbparamstat[0];k++) paramstat[k] = parstatmultvec[cmpt+k];
 	cmpt = cmpt + nbparamstat[0];
 	compquant(n,lawindex,stat,M,statvec,//lawname,statname,
-		  nbparlaw,parlaw,nbparamstat,paramstat,modelnum,funclist,thetavec,xvec,p,np);
+		  nbparlaw,parlaw,nbparamstat,paramstat,modelnum,funclist,thetavec,xvec,p,np,center,scale);
 	delete[] paramstat;
       } else {
 	compquant(n,lawindex,stat,M,statvec,// lawname,statname,
-		  nbparlaw,parlaw,nbparamstat,(double*)0,modelnum,funclist,thetavec,xvec,p,np);
+		  nbparlaw,parlaw,nbparamstat,(double*)0,modelnum,funclist,thetavec,xvec,p,np,center,scale);
       }
 
       for (j=0;j<M[0];j++) liststat[i*M[0] + j] = statvec[j];
@@ -838,7 +875,7 @@ int *nbparlaw, double *parlaw, int *nbparstat, double *parstat, int *modelnum, c
 
     char **name;          // name = rep(" ", 50)
     name = new char*[50];
-    for (i=0;i<50;i++) {
+    for (i = 0; i < 50; i++) {
       name[i] =  new char[1];
       name[i][0] = ' ';
     }
@@ -881,7 +918,7 @@ int *nbparlaw, double *parlaw, int *nbparstat, double *parstat, int *modelnum, c
 
     for (j=0;j<N[0];j++) {
 
-      gensample(nulldist[0],n,x,name,getname,params,nbparams,setseed);  // We retrieve x
+      gensample(nulldist[0],n,x,name,getname,params,nbparams,setseed,center,scale);  // We retrieve x
 
       cmpt = 0;
       for (i=0;i<nbstats[0];i++) {
